// electronic fuse using ACS712-5A which outputs 2.5V when no current is flowing, and 185mV per amp so +3A would output 3055mV 
// LED is a dual colour 2 lead LED. 
// When fuse trips it will stay tripped until reset button is pressed.
// With 16VAC transformer this theoretically could provide 5V at 3A, 9V at 2.5 but I have set the overcurrent to trip around 2.25A.
// Designed for ATtiny85 using damellis's ATtinycore fork https://raw.githubusercontent.com/damellis/attiny/ide-1.6.x-boards-manager/package_damellis_attiny_index.json

#include "Button.h"

#define F_CPU 16000000UL 

int greenLED = 0;
int redLED = 1;
int relay = 3;
// pin2 (physical pin 7) is analog input 1 on the ATtiny85 used for current sensor

float current = 0.00;
boolean enable = false;
boolean overloaded = false;

Button enable_SW = Button(4,BUTTON_PULLUP_INTERNAL); 

void setup() 
{
  
  analogReference(DEFAULT);
  pinMode(greenLED, OUTPUT);
  pinMode(redLED, OUTPUT);
  pinMode(relay, OUTPUT);
  digitalWrite(greenLED,LOW);
  digitalWrite(redLED,LOW);
  digitalWrite(relay,LOW);
}

void loop()
{
  current = (analogRead(1) * 5.05) / 1023.0;  // return current as a voltage from analog A1 **change 5.05 to measured VCC**
  
  if(enable_SW.uniquePress() && overloaded == false)
    {
    if(enable)
    {
      digitalWrite(redLED,LOW);
      digitalWrite(greenLED, HIGH); 
      digitalWrite(relay, HIGH);       
      enable = !enable;
    }
    else
    {
      digitalWrite(greenLED, LOW); 
      digitalWrite(relay, LOW);  
      enable = !enable;
    }
  }
  
  delay(50);  
  
  if (current >=2.87 || current <=2.13) // 2870 = 2A, 2916mV = 2.25A, 2962mV = 2.5A, 3055mV = 3A (overcurrent trip set at 2A)

    {
       digitalWrite(greenLED,LOW);
       digitalWrite(relay,LOW);
       digitalWrite(redLED,HIGH);
       overloaded = true; // make it so that cannot be turned on unless circuit is reset or overload clears.
       delay(1000);
    }

    else if (current <= 2.82 || current <=2.17) // approx 1.75A can be supplied continuously

    {
      overloaded = false; // set overload flag to false but leave output off after overload until enable button is pressed again.
    }

 
}

/// end of program ///
